/* c-irrlicht --- C bindings for Irrlicht Engine

   Copyright (C) 2019 Javier Sancho <jsf@jsancho.org>

   This file is part of c-irrlicht.

   c-irrlicht is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 3 of the
   License, or (at your option) any later version.

   c-irrlicht is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with guile-irrlicht.  If not, see
   <http://www.gnu.org/licenses/>.
*/

#include <irrlicht/irrlicht.h>
#include "ISceneManager.h"

extern "C" {
  irr_scene_IAnimatedMeshSceneNode*
  irr_scene_addAnimatedMeshSceneNode(irr_scene_ISceneManager* smgr,
                                     irr_scene_IAnimatedMesh* mesh,
                                     irr_scene_ISceneNode* parent,
                                     int id,
                                     irr_core_vector3df* position,
                                     irr_core_vector3df* rotation,
                                     irr_core_vector3df* scale,
                                     int alsoAddIfMeshPointerZero)
  {
    const irr::core::vector3df iPosition =
      position ?
      irr::core::vector3df(position->x, position->y, position->z) :
      irr::core::vector3df(0, 0, 0);

    const irr::core::vector3df iRotation =
      rotation ?
      irr::core::vector3df(rotation->x, rotation->y, rotation->z) :
      irr::core::vector3df(0, 0, 0);

    const irr::core::vector3df& iScale =
      scale ?
      irr::core::vector3df(scale->x, scale->y, scale->z) :
      irr::core::vector3df(1, 1, 1);

    return (irr_scene_IAnimatedMeshSceneNode*)
      ((irr::scene::ISceneManager*)smgr)
      ->addAnimatedMeshSceneNode((irr::scene::IAnimatedMesh*)mesh,
                                 (irr::scene::ISceneNode*)parent,
                                 id,
                                 iPosition,
                                 iRotation,
                                 iScale,
                                 alsoAddIfMeshPointerZero);
  }

  irr_scene_ICameraSceneNode*
  irr_scene_addCameraSceneNode(irr_scene_ISceneManager* smgr,
                               irr_scene_ISceneNode* parent,
                               irr_core_vector3df* position,
                               irr_core_vector3df* lookat,
                               int id,
                               int makeActive)
  {
    const irr::core::vector3df& iPosition =
      position ?
      irr::core::vector3df(position->x, position->y, position->z) :
      irr::core::vector3df(0, 0, 0);

    const irr::core::vector3df& iLookat =
      lookat ?
      irr::core::vector3df(lookat->x, lookat->y, lookat->z) :
      irr::core::vector3df(0, 0, 100);

    return (irr_scene_ICameraSceneNode*)
      ((irr::scene::ISceneManager*)smgr)
      ->addCameraSceneNode((irr::scene::ISceneNode*)parent,
                           iPosition,
                           iLookat,
                           id,
                           makeActive);
  }

  void
  irr_scene_drawAll(irr_scene_ISceneManager* smgr)
  {
    ((irr::scene::ISceneManager*)smgr)->drawAll();
  }


  irr_scene_IAnimatedMesh*
  irr_scene_getMesh(irr_scene_ISceneManager* smgr,
                    const char* filename)
  {
    return (irr_scene_IAnimatedMesh*)
      ((irr::scene::ISceneManager*)smgr)->getMesh(filename);
  }
}
